/*
Module : APIPE.CPP
Purpose: Defines the implementation for an MFC wrapper class
         for Win32 anonymous Pipes
Created: PJN / 2-08-1998
History: None

Copyright (c) 1998 by PJ Naughter.  
All rights reserved.

*/


/////////////////////////////////  Includes  //////////////////////////////////
#include "stdafx.h"
#include "apipe.h"



#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


///////////////////////////// CAnonymousPipe Implementation ///////////////////

IMPLEMENT_DYNAMIC(CAnonymousPipe, CObject)

CAnonymousPipe::CAnonymousPipe()
{
  m_hWrite = INVALID_HANDLE_VALUE;
  m_hRead = INVALID_HANDLE_VALUE;
}

CAnonymousPipe::~CAnonymousPipe()
{
  Close();
}

BOOL CAnonymousPipe::Create(LPSECURITY_ATTRIBUTES lpPipeAttributes, DWORD dwSize)
{
  ASSERT(!IsOpen());

  BOOL bSuccess = ::CreatePipe(&m_hRead, &m_hWrite, lpPipeAttributes, dwSize);
  if (!bSuccess)
    TRACE1("CAnonymousPipe::Create() failed, GetLastError returned %d\n", ::GetLastError());

  return bSuccess;
}

BOOL CAnonymousPipe::Write(LPCVOID lpBuffer, DWORD dwNumberOfBytesToWrite, DWORD& dwNumberOfBytesWritten)
{
  ASSERT(m_hWrite != NULL); //Pipe must be open

  BOOL bSuccess = ::WriteFile(m_hWrite, lpBuffer, dwNumberOfBytesToWrite, 
                              &dwNumberOfBytesWritten, NULL);
  if (!bSuccess)
    TRACE1("CAnonymousPipe::Write() failed, GetLastError returned %d\n", ::GetLastError());

  return bSuccess;
}

BOOL CAnonymousPipe::Read(LPVOID lpBuffer, DWORD dwNumberOfBytesToRead, DWORD& dwNumberOfBytesRead)
{
  ASSERT(m_hRead != NULL); //Pipe must be open

  BOOL bSuccess = ::ReadFile(m_hRead, lpBuffer, dwNumberOfBytesToRead, 
                             &dwNumberOfBytesRead, NULL);
  if (!bSuccess)
    TRACE1("CAnonymousPipe::Read() failed, GetLastError returned %d\n", ::GetLastError());

  return bSuccess;
}

BOOL CAnonymousPipe::Peek(LPVOID lpBuffer, DWORD dwBufferSize, DWORD& dwBytesRead, 
                          DWORD& dwTotalBytesAvail, DWORD& dwBytesLeftThisMessage)
{
  ASSERT(m_hRead != NULL);

  BOOL bSuccess = ::PeekNamedPipe(m_hRead, lpBuffer, dwBufferSize, &dwBytesRead, 
                                  &dwTotalBytesAvail, &dwBytesLeftThisMessage);
  if (!bSuccess)
    TRACE1("CNamedPipe::Peek() failed, GetLastError returned %d\n", ::GetLastError());

  return bSuccess;
}

BOOL CAnonymousPipe::Close()
{
  BOOL bSuccess = TRUE;
  if (IsOpen())
  {
    BOOL bSuccess1 = CloseHandle(m_hWrite);
    if (!bSuccess1)
      TRACE1("CAnonymousPipe::Close() failed, GetLastError returned %d\n", GetLastError());

    BOOL bSuccess2 = CloseHandle(m_hRead);
    if (!bSuccess2)
      TRACE1("CAnonymousPipe::Close() failed, GetLastError returned %d\n", GetLastError());

    bSuccess = bSuccess1 && bSuccess2;
  }

  return bSuccess;
}

#ifdef _DEBUG
void CAnonymousPipe::AssertValid() const
{
  CObject::AssertValid();
  ASSERT(IsOpen());
}
#endif

#ifdef _DEBUG
void CAnonymousPipe::Dump(CDumpContext& dc) const
{
  CObject::Dump(dc);

  CString sText;
  sText.Format(_T("Open=%d, Write Handle=%x, Read Handle=%x\n"), IsOpen(), m_hWrite, m_hRead);
  dc << sText;
}
#endif

